<?php
/**
 * This Software is the property of Data Development and is protected
 * by copyright law - it is NOT Freeware.
 * Any unauthorized use of this software without a valid license
 * is a violation of the license agreement and will be prosecuted by
 * civil and criminal law.
 * http://www.shopmodule.com
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author        D3 Data Development - Daniel Seifert <support@shopmodule.com>
 * @link          http://www.oxidmodule.com
 */

/**
 * Class d3_cfg_watermark_Main
 */
class d3_cfg_watermark_Main extends d3_cfg_mod_main
{

    protected $_sThisTemplate = 'd3_cfg_watermark_main.tpl';

    protected $_sModId = 'd3_watermark';

    protected $_sMenuItemTitle = 'd3mxwatermark';

    protected $_sMenuSubItemTitle = 'd3mxwatermark_settings';

    protected $_sWatermarkName = 'd3_watermark';

    protected $_aImageTypes = array("GIF" => IMAGETYPE_GIF, "JPG" => IMAGETYPE_JPEG, "PNG" => IMAGETYPE_PNG);

    protected $_sHelpLinkMLAdd = 'D3_WATERMARK_MAIN_HELPLINK';

    protected $_blFailUpload = false;

    protected $_blNewWatermark = false;

    protected $_aAllowExtension = array("gif", "jpg", "png");

//    protected $_blHasDebugSwitch = false;
//    protected $_blHasTestModeSwitch = false;
//    protected $_sDebugHelpTextIdent = 'D3_CFG_MOD_GENERAL_DEBUGACTIVE_DESC';
//    protected $_sTestModeHelpTextIdent = 'D3_CFG_MOD_GENERAL_TESTMODEACTIVE_DESC';

    /**
     * @var d3clrtmp
     */
    public $oClrTmp;

    /**
     * @return string
     */
    public function render()
    {
        //Fehler beim Upload?
        $this->addTplParam("isfailupload", $this->_blFailUpload);
        $this->addTplParam("isNewWatermarkAdded", $this->getHasNewWatermarkPicture());
        $this->addTplParam("sNewSettingsMessage", 'D3_WATERMARK_NEW_WATERMARKPICTURE');
        $this->setHasNewWatermarkPicture();

        // knnen generierte Bilder via Connector entfernt werden?
        $blShowDeleteGeneratedPix = false;
        if(method_exists($this->_getClrTmp(), 'clearGeneratedImgs')){
            $blShowDeleteGeneratedPix = true;
        }
        $this->addTplParam("blShowDeleteGeneratedPix", $blShowDeleteGeneratedPix);

        return parent::render();
    }

    /**
     * @param bool $blNew
     */
    public function setHasNewWatermarkPicture($blNew = false)
    {
        $this->_blNewWatermark = $blNew;
    }

    /**
     * @return bool
     */
    public function getHasNewWatermarkPicture()
    {
        return $this->_blNewWatermark;
    }

    public function save()
    {
        $this->_savePicture();

        $generellSettings = oxRegistry::getConfig()->getRequestParameter('value');
        d3_cfg_mod::get('d3_watermark')->d3getLog()->log(
            d3log::INFO,
            __CLASS__,
            __FUNCTION__,
            __LINE__,
            "save generell settings via parent call",
            "settings: " . var_export($generellSettings, true)
        );

        parent::save();

        return;
    }

    protected function _savePicture()
    {
        $sFilename = $_FILES['imagefile']['name'];

        if (false == empty($sFilename)) {
            /** @var d3filesystem $oFS */
            $oFS               = oxNew('d3filesystem');
            $aFile             = $oFS->splitFilename($sFilename);
            $sCompareExtension = strtolower($aFile['ext']);

            if (false == in_array($sCompareExtension, $this->_aAllowExtension)) {
                d3_cfg_mod::get('d3_watermark')->d3getLog()->log(
                    d3log::ERROR,
                    __CLASS__,
                    __FUNCTION__,
                    __LINE__,
                    "wrong file extension",
                    "upload failed for file " . $sFilename . "\n set fail upload and return"
                );

                $this->_blFailUpload = 11;

                return false;
            }

            $sFileName = $this->getWatermarkName($sFilename);
            $sDest     = $this->getConfig()->getImageDir($this->isAdmin()) . $sFileName; // ist aktuell immer out/admin/img/d3_watermark.png

            if (@ copy($_FILES['imagefile']['tmp_name'], $sDest)) {
                if (move_uploaded_file($_FILES['imagefile']['tmp_name'], $sDest)) {
                    $sType = $this->_createWMIcon($sDest, $sFileName);

                    $this->d3GetSet()->setValue('sWaterMark_FileName', $sFileName);
                    $this->d3GetSet()->setValue('sWaterMark_FileType', $sType);
                    $this->setHasNewWatermarkPicture(true);

                    d3_cfg_mod::get('d3_watermark')->d3getLog()->log(
                        d3log::INFO,
                        __CLASS__,
                        __FUNCTION__,
                        __LINE__,
                        "new file is saved",
                        "file is saved to " . $sDest
                    );

                    // clear cached icon, show new watermark pic in admin

                }
            }
        } else {
            d3_cfg_mod::get('d3_watermark')->d3getLog()->log(
                d3log::DEBUG,
                __CLASS__,
                __FUNCTION__,
                __LINE__,
                "no new file found",
                var_export($sFilename, true)
            );
        }
    }

    /**
     * @param $sSourceImage
     * @param $sFileName
     *
     * @return mixed
     */
    protected function _createWMIcon($sSourceImage, $sFileName)
    {
        list($width, $height, $type) = getimagesize($sSourceImage);

        $pDestImage   = null;
        $pSourceImage = null;
        $iNewWidth    = 100;
        $iFactor      = $width / $iNewWidth;
        $iNewHeight   = round($height / $iFactor);

        switch ($type) {
            case ($this->_aImageTypes["GIF"]):
                $pSourceImage = imagecreatefromgif($sSourceImage);
                $pDestImage   = imagecreate($iNewWidth, $iNewHeight);
                break;
            case ($this->_aImageTypes["JPG"]):
                $pSourceImage = imagecreatefromjpeg($sSourceImage);
                $pDestImage   = imagecreatetruecolor($iNewWidth, $iNewHeight);
                break;
            case ($this->_aImageTypes["PNG"]):
                $pSourceImage = imagecreatefrompng($sSourceImage);
                $pDestImage   = imagecreatetruecolor($iNewWidth, $iNewHeight);
                break;
        }

        $sIcoFileName      = $this->getWatermarkName($sFileName, '_ico');
        $sDestinationImage = $this->getConfig()->getImageDir(1) . $sIcoFileName;

        $imgWhite = imagecolorallocate($pDestImage, 255, 255, 255);
        imagefill($pDestImage, 0, 0, $imgWhite);
        imagecolortransparent($pDestImage, $imgWhite);

        if (imagecopyresized($pDestImage, $pSourceImage, 0, 0, 0, 0, $iNewWidth, $iNewHeight, $width, $height)) {
            switch ($type) {
                case ($this->_aImageTypes["GIF"]):
                    imagegif($pDestImage, $sDestinationImage);
                    break;
                case ($this->_aImageTypes["JPG"]):
                    imagejpeg($pDestImage, $sDestinationImage);
                    break;
                case ($this->_aImageTypes["PNG"]):
                    imagepng($pDestImage, $sDestinationImage);
                    break;
            }

            $this->d3GetSet()->setValue('sWaterMark_IcoFileName', $sIcoFileName);
        }

        return $type;
    }

    /**
     *
     */
    public function d3deleteallGeneratedAticlePitures()
    {
        /** @var $oClrTmp d3clrtmp */
        $oClrTmp = $this->_getClrTmp();
        $oClrTmp->clearGeneratedImgs();

        oxRegistry::get("oxUtilsView")->addErrorToDisplay('D3_CFG_CLRTMP_GENIMGS_SUCC');
    }

    /**
     * @return d3clrtmp
     */
    protected function _getClrTmp()
    {
        if (false == $this->oClrTmp) {
            $this->oClrTmp = oxNew('d3clrtmp');
        }

        return $this->oClrTmp;
    }


    public function getWatermarkUrl()
    {
        $oViewConfig = $this->getViewConfig();
        $pictureName = d3_cfg_mod::get('d3_watermark')->getEditValue('sWaterMark_IcoFileName');
        $picturePAth = $this->getConfig()->getImagePath($pictureName, true);

        return $oViewConfig->getImageUrl($pictureName). '?'.filemtime($picturePAth);
    }

    /**
     * @param string $sFilename
     * @param string $additionName
     *
     * @return string
     */
    protected function getWatermarkName($sFilename, $additionName = '')
    {
        $sFileName = $this->_sWatermarkName;
        $sFileName .= '_'. $this->getConfig()->getActiveShop()->getShopId();
        $sFileName .= $additionName;
        $sFileName .= substr($sFilename, strrpos($sFilename, '.'));

        return $sFileName;
    }
}
